#include "settings.h"
#include "settings.moc"

#include <qlabel.h>
#include <qhbox.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qhgroupbox.h>

#include <klocale.h>
#include <knuminput.h>
#include <kcolorbutton.h>
#include <kapplication.h>
#include <kdialogbase.h>

#include "piece.h"
#include "factory.h"


//-----------------------------------------------------------------------------
ConfigGroupSaver::ConfigGroupSaver()
    : KConfigGroupSaver(kapp->config(), "Options")
{}

//-----------------------------------------------------------------------------
BaseAppearanceConfig::BaseAppearanceConfig()
    : QWidget(0, "appearance_config")
{
    const BaseBoardInfo &bbi = BaseFactory::self()->bbi;
    QVBoxLayout *top = new QVBoxLayout(this);

    // upper part
    _main = new QWidget(this);
    top->addWidget(_main);
    _grid = new QGridLayout(_main, 3, 2, 0, KDialog::spacingHint());

    QLabel *label = new QLabel(i18n("Block size:"), _main);
    _grid->addWidget(label, 0, 0);
    KIntNumInput *in = new KIntNumInput(_main, "block size");
    in->setRange(bbi.minBlockSize, bbi.maxBlockSize);
    in->setValue(bbi.defaultBlockSize);
    _grid->addWidget(in, 0, 1);

    _grid->addRowSpacing(1, KDialog::spacingHint());

    QCheckBox *chb =
        new QCheckBox(i18n("Enable animations."), _main, "enable animations");
    chb->setChecked(true);
    _grid->addMultiCellWidget(chb, 2, 2, 0, 1);

    top->addSpacing(KDialog::spacingHint());

    // lower part
    QHGroupBox *gbox = new QHGroupBox(i18n("Background"), this);
    top->addWidget(gbox);
    QWidget *widget = new QWidget(gbox);
    QGridLayout *grid =
        new QGridLayout(widget, 2, 3, 0, KDialog::spacingHint());
    label = new QLabel(i18n("Fade strength:"), widget);
    grid->addWidget(label, 0, 0);
    KDoubleNumInput *dn = new KDoubleNumInput(widget, "fade intensity");
    dn->setRange(0.0, 1.0, 0.01);
    dn->setValue(1.0);
    grid->addMultiCellWidget(dn, 0, 0, 1, 2);
    label = new QLabel(i18n("Fade color:"), widget);
    grid->addWidget(label, 1, 0);
    KColorButton *cob = new KColorButton(widget, "fade color");
    cob->setColor(black);
    grid->addWidget(cob, 1, 1);

    top->addStretch(1);
}

uint BaseAppearanceConfig::blockSize()
{
    const BaseBoardInfo &bbi = BaseFactory::self()->bbi;
    ConfigGroupSaver cg;
    uint s =
        cg.config()->readUnsignedNumEntry("block size", bbi.defaultBlockSize);
    return kMax(bbi.minBlockSize, kMin(bbi.maxBlockSize, s));
}

double BaseAppearanceConfig::fadeIntensity()
{
    ConfigGroupSaver cg;
    return cg.config()->readDoubleNumEntry("fade intensity", 1.0);
}

QColor BaseAppearanceConfig::fadeColor()
{
    ConfigGroupSaver cg;
    QColor color(black);
    return cg.config()->readColorEntry("fade color", &color);
}

bool BaseAppearanceConfig::isAnimationsEnabled()
{
    ConfigGroupSaver cg;
    return cg.config()->readBoolEntry("enable animations", true);
}

bool BaseAppearanceConfig::isMenubarVisible()
{
    ConfigGroupSaver cg;
    return cg.config()->readBoolEntry("menubar visible", true);
}

void BaseAppearanceConfig::saveMenubarVisible(bool visible)
{
    ConfigGroupSaver cg;
    cg.config()->writeEntry("menubar visible", visible);
}


//-----------------------------------------------------------------------------
ColorConfig::ColorConfig()
    : QWidget(0, "color_config")
{
    const GPieceInfo &info = Piece::info();
    QVBoxLayout *top = new QVBoxLayout(this);
    uint nb = info.nbColors();
    QGridLayout *grid = new QGridLayout(top, nb+1, 3, KDialog::spacingHint());
    grid->setColStretch(2, 1);
    for (uint i=0; i<nb; i++) {
        QLabel *label = new QLabel(info.colorLabel(i), this);
        grid->addWidget(label, i, 0);
        KColorButton *cob = new KColorButton(this, colorKey(i));
        cob->setFixedWidth(100);
        cob->setColor(info.defaultColor(i));
        grid->addWidget(cob, i, 1);
    }
    grid->setRowStretch(nb, 1);
}

QCString ColorConfig::colorKey(uint i)
{
    QCString s;
    s.setNum(i);
    return "color #" + s;
}

QColor ColorConfig::color(uint i)
{
    ConfigGroupSaver cg;
    QColor color = Piece::info().defaultColor(i);
    return cg.config()->readColorEntry(colorKey(i).data(), &color);
}

